/*
 * Oncilla.h
 *
 *  Created on: 15 dec. 2011
 *      Author: Alexandre Tuleu, Arne Nordmann
 */
#pragma once

#include <vector>
#include <iostream>
#include <boost/shared_ptr.hpp>

#include "common.h"
#include "OncillaSynchronizer.h"


namespace rci {
namespace oncilla {


class Oncilla;
typedef boost::shared_ptr<Oncilla> OncillaPtr;

class OncillaSynchronizer;
typedef boost::shared_ptr<OncillaSynchronizer> OncillaSynchronizerPtr;

class Oncilla {
public:

	static const std::string & nameOfLeg(Leg l);

	Oncilla();
	virtual ~Oncilla();

	OncillaL0Ptr getL0(Leg l) const;
	OncillaL1Ptr getL1(Leg l) const;
	OncillaL2Ptr getL2(Leg l) const;
	OncillaL3Ptr getL3(Leg l) const;
	OncillaL4Ptr getL4(Leg l) const;
	OncillaTrunkPtr getTrunk() const;

	OncillaSynchronizerPtr getSynchronizer() const;


private:
	typedef std::vector<std::string> LegNames;

	static LegNames s_legNames;

	void init();

	static OncillaSynchronizerPtr s_synchronizer;

	std::vector<OncillaL0Ptr> d_L0s;
	std::vector<OncillaL1Ptr> d_L1s;
	std::vector<OncillaL2Ptr> d_L2s;
	std::vector<OncillaL3Ptr> d_L3s;
	std::vector<OncillaL4Ptr> d_L4s;

	OncillaTrunkPtr d_trunk;
};

}
}
