#pragma once

#include <iostream>
#include <boost/shared_ptr.hpp>

#include <rci/ResourceNode.h>
#include <rci/Controlled.h>
#include <rci/Sensing.h>

#include <rci/dto/JointAngles.h>


namespace rci {
namespace oncilla {

class OncillaL0;
typedef boost::shared_ptr<OncillaL0> OncillaL0Ptr;

/**
 * Node class, representing the hip node of the quadruped robot.
 * @todo In case of simulation, this node can also sense the power consumption.
 */
class OncillaL0: public rci::ResourceNode,
                 public rci::Controlled,
                 public rci::PositionControlled {

public:

	/**
	 * Special constructor to also link to webots
	 */
	OncillaL0(const std::string &name = "Oncilla Hip");

	virtual ~OncillaL0();

	/**
	 * Returns, if controller is converged.
	 * @return True, if controller is converged after last command.
	 */
	bool isConverged() const;

	/**
	 * Commanding a joint position.
	 * @param position Position command
	 * @return Return, if successfull. (e.g. not exceeding joint limits)
	 * @todo Check for correct control mode
	 */
	bool setJointPosition(JointAnglesPtr position);

	void unsafeSetCommand(double value);

	double unsafeGetCommand() const;

	/**
	 * Returns latest position command. Note, that this is the latest valid
	 * commanded position. If the position command exceeds limits and is
	 * therefore rejected, it ..
	 */
	virtual JointAnglesPtr getLastPositionCommand() const;

	/**
	 * Print
	 */
	std::string print() const;
};

}
}
