#pragma once

#include <iostream>
#include <boost/shared_ptr.hpp>

#include <rci/ResourceNode.h>
#include <rci/Controlled.h>
#include <rci/Configurable.h>
#include <rci/Sensing.h>

#include <rci/dto/JointAngles.h>

namespace rci {
namespace oncilla {

class OncillaL3;
typedef boost::shared_ptr<OncillaL3> OncillaL3Ptr;

/**
 * Node class, representing the foot/ankle of the quadruped robot. Not actuated,
 * but force sensing.
 */
class OncillaL3: public rci::ResourceNode,
                 public rci::Sensing,
                 public rci::PositionSensing {

public:

	OncillaL3(const std::string & name = "Oncilla Ankle");

	virtual ~OncillaL3();

	/**
	 * Returns contact forces
	 */
	JointAnglesPtr getJointPosition() const;


	double unsafeGetPosition() const;

	void unsafeUpdatePosition(double value);
	/**
	 * Print
	 */
	std::string print() const;
};

}
}
