#pragma once

#include <iostream>
#include <boost/shared_ptr.hpp>

#include <rci/ResourceNode.h>
#include <rci/Controlled.h>
#include <rci/Configurable.h>
#include <rci/Sensing.h>

#include <rci/dto/JointAngles.h>

namespace rci {
namespace oncilla {

class OncillaL4;
typedef boost::shared_ptr<OncillaL4> OncillaL4Ptr;

/**
 * Node class, representing the foot of the quadruped robot. Not actuated,
 * but force sensing.
 */
class OncillaL4: public rci::ResourceNode,
                 public rci::Sensing,
                 public rci::ForceSensing,
                 public rci::TorqueSensing{

public:

	OncillaL4(const std::string & name = "Oncilla Foot");

	virtual ~OncillaL4();

	/**
	 * Returns contact forces
	 */
	ForcesPtr getForces() const;

	/**
	 * Print
	 */
	std::string print() const;


	void unsafeUpdateForce(const double * data);
};

}
}
