#pragma once

#include <iostream>
#include <boost/shared_ptr.hpp>

#include <rci/ResourceNode.h>
#include <rci/Controlled.h>
#include <rci/Configurable.h>
#include <rci/Sensing.h>

namespace rci {
namespace oncilla {

class OncillaTrunk;
typedef boost::shared_ptr<OncillaTrunk> OncillaTrunkPtr;

/**
 * Node class, representing the quadruped trunk, including pose sensing.
 *
 * @todo Expose translational accelerations and rotational velocities
 */
class OncillaTrunk: public rci::ResourceNode, public Sensing, public PoseSensing {

public:

    OncillaTrunk(const std::string & name = "Oncilla Trunk");
    virtual ~OncillaTrunk();

    /**
     * Returns current pose.
     *
     * @return Current pose
     */
    virtual PosePtr getAbsolutePose() const;

    /**
     * Returns current displacement.
     *
     * @return Current pose
     */
    virtual PosePtr getDisplacement() const;

    /**
     * Returns current translational and rotational acceleration.
     *
     * @return Current pose
     */
    virtual PosePtr getAcceleration() const;

    /**
     * Print
     */
    std::string print() const;

};

}
}
